import { Mapper } from '@automapper/core';
import { InjectMapper } from '@automapper/nestjs';
import { Controller, UsePipes } from '@nestjs/common';
import { CommandBus, QueryBus } from '@nestjs/cqrs';
import { GrpcMethod, RpcException } from '@nestjs/microservices';
import { DatabaseException } from 'src/modules/database/src/exceptions/DatabaseException';
import { AddUsernameCommand } from '../../commands/add-username.command';
import { CreateAuthCommand } from '../../commands/create-auth.command';
import { DeleteAuthCommand } from '../../commands/delete-auth.command';
import { DeleteUsernameCommand } from '../../commands/delete-username.command';
import { UpdatePasswordCommand } from '../../commands/update-password.command';
import { UpdateUsernameCommand } from '../../commands/update-username.command';
import { AddUsernameRequest } from '../../domain/dtos/add-username.request';
import { CreateAuthRequest } from '../../domain/dtos/create-auth.request';
import { DeleteAuthRequest } from '../../domain/dtos/delete-auth.request';
import { DeleteUsernameRequest } from '../../domain/dtos/delete-username.request';
import { UpdatePasswordRequest } from '../../domain/dtos/update-password.request';
import { UpdateUsernameRequest } from '../../domain/dtos/update-username.request';
import { ValidateAuthRequest } from '../../domain/dtos/validate-auth.request';
import { Auth } from '../../domain/entities/auth';
import { Username } from '../../domain/entities/username';
import { ValidateAuthQuery } from '../../queries/validate-auth.query';
import { AuthPresenter } from './auth.presenter';
import { RpcValidationPipe } from './rpc.validation-pipe';
import { UsernamePresenter } from './username.presenter';

@UsePipes(
  new RpcValidationPipe({
    whitelist: true,
    forbidUnknownValues: false,
  }),
)
@Controller()
export class AuthController {
  constructor(
    private readonly _commandBus: CommandBus,
    private readonly _queryBus: QueryBus,
    @InjectMapper() private readonly _mapper: Mapper,
  ) {}

  @GrpcMethod('AuthService', 'Validate')
  async validate(data: ValidateAuthRequest): Promise<AuthPresenter> {
    try {
      const auth: Auth = await this._queryBus.execute(
        new ValidateAuthQuery(data.username, data.password),
      );
      return this._mapper.map(auth, Auth, AuthPresenter);
    } catch (e) {
      throw new RpcException({
        code: 7,
        message: 'Permission denied',
      });
    }
  }

  @GrpcMethod('AuthService', 'Create')
  async createUser(data: CreateAuthRequest): Promise<AuthPresenter> {
    try {
      const auth: Auth = await this._commandBus.execute(
        new CreateAuthCommand(data),
      );
      return this._mapper.map(auth, Auth, AuthPresenter);
    } catch (e) {
      if (e instanceof DatabaseException) {
        if (e.message.includes('Already exists')) {
          throw new RpcException({
            code: 6,
            message: 'Auth already exists',
          });
        }
      }
      throw new RpcException({
        code: 7,
        message: 'Permission denied',
      });
    }
  }

  @GrpcMethod('AuthService', 'AddUsername')
  async addUsername(data: AddUsernameRequest): Promise<UsernamePresenter> {
    try {
      const username: Username = await this._commandBus.execute(
        new AddUsernameCommand(data),
      );

      return this._mapper.map(username, Username, UsernamePresenter);
    } catch (e) {
      if (e instanceof DatabaseException) {
        if (e.message.includes('Already exists')) {
          throw new RpcException({
            code: 6,
            message: 'Username already exists',
          });
        }
      }
      throw new RpcException({
        code: 7,
        message: 'Permission denied',
      });
    }
  }

  @GrpcMethod('AuthService', 'UpdateUsername')
  async updateUsername(
    data: UpdateUsernameRequest,
  ): Promise<UsernamePresenter> {
    try {
      const username: Username = await this._commandBus.execute(
        new UpdateUsernameCommand(data),
      );

      return this._mapper.map(username, Username, UsernamePresenter);
    } catch (e) {
      if (e instanceof DatabaseException) {
        if (e.message.includes('Already exists')) {
          throw new RpcException({
            code: 6,
            message: 'Username already exists',
          });
        }
      }
      throw new RpcException({
        code: 7,
        message: 'Permission denied',
      });
    }
  }

  @GrpcMethod('AuthService', 'UpdatePassword')
  async updatePassword(data: UpdatePasswordRequest): Promise<AuthPresenter> {
    try {
      const auth: Auth = await this._commandBus.execute(
        new UpdatePasswordCommand(data),
      );

      return this._mapper.map(auth, Auth, AuthPresenter);
    } catch (e) {
      throw new RpcException({
        code: 7,
        message: 'Permission denied',
      });
    }
  }

  @GrpcMethod('AuthService', 'DeleteUsername')
  async deleteUsername(data: DeleteUsernameRequest) {
    try {
      return await this._commandBus.execute(new DeleteUsernameCommand(data));
    } catch (e) {
      throw new RpcException({
        code: 7,
        message: 'Permission denied',
      });
    }
  }

  @GrpcMethod('AuthService', 'Delete')
  async deleteAuth(data: DeleteAuthRequest) {
    try {
      return await this._commandBus.execute(new DeleteAuthCommand(data));
    } catch (e) {
      throw new RpcException({
        code: 7,
        message: 'Permission denied',
      });
    }
  }
}
