import { classes } from '@automapper/classes';
import { AutomapperModule } from '@automapper/nestjs';
import { Test, TestingModule } from '@nestjs/testing';
import { AuthRepository } from '../../adapters/secondaries/auth.repository';
import { CreateAuthCommand } from '../../commands/create-auth.command';
import { CreateAuthRequest } from '../../domain/dtos/create-auth.request';
import { Auth } from '../../domain/entities/auth';
import { CreateAuthUseCase } from '../../domain/usecases/create-auth.usecase';
import * as bcrypt from 'bcrypt';
import { UsernameRepository } from '../../adapters/secondaries/username.repository';
import { Type } from '../../domain/dtos/type.enum';
import { LoggingMessager } from '../../adapters/secondaries/logging.messager';

const newAuthRequest: CreateAuthRequest = {
  uuid: 'bb281075-1b98-4456-89d6-c643d3044a91',
  username: 'john.doe@email.com',
  password: 'John123',
  type: Type.EMAIL,
};
const newAuthCommand: CreateAuthCommand = new CreateAuthCommand(newAuthRequest);

const mockAuthRepository = {
  create: jest.fn().mockResolvedValue({
    uuid: newAuthRequest.uuid,
    password: bcrypt.hashSync(newAuthRequest.password, 10),
  }),
};

const mockUsernameRepository = {
  create: jest.fn().mockResolvedValue({
    uuid: newAuthRequest.uuid,
    username: newAuthRequest.username,
    type: newAuthRequest.type,
  }),
};

const mockMessager = {
  publish: jest.fn().mockImplementation(),
};

describe('CreateAuthUseCase', () => {
  let createAuthUseCase: CreateAuthUseCase;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [AutomapperModule.forRoot({ strategyInitializer: classes() })],
      providers: [
        {
          provide: AuthRepository,
          useValue: mockAuthRepository,
        },
        {
          provide: UsernameRepository,
          useValue: mockUsernameRepository,
        },
        {
          provide: LoggingMessager,
          useValue: mockMessager,
        },
        CreateAuthUseCase,
      ],
    }).compile();

    createAuthUseCase = module.get<CreateAuthUseCase>(CreateAuthUseCase);
  });

  it('should be defined', () => {
    expect(createAuthUseCase).toBeDefined();
  });

  describe('execute', () => {
    it('should create an auth with an encrypted password', async () => {
      const newAuth: Auth = await createAuthUseCase.execute(newAuthCommand);

      expect(
        bcrypt.compareSync(newAuthRequest.password, newAuth.password),
      ).toBeTruthy();
    });
  });
});
